var $ = jQuery;
$(function() {
    const DisciplineID = g_disciplineID;
    const FacultyID = g_facultyID;

    $('div.GroupContainerAttached').show().each(function() {
        $(this).css('height', $(this).height());
    });
    let $studentInput = $('.InputStudentName');
    let $gradeSelect = $('.SelectGrade');
    let $groupSelect = $('.SelectStudyGroup');
    let $searchResult = $('.SearchResult');
    let $attStudentList = $('div.AttachedStudentsList');
    let $studentsList = $('.StudentsList');
    let $generalStudentsList = $('.GeneralStudentsList');


    /**
     * @param {object} group
     * @param {int} group.GroupID
     * @param {int} group.GroupNum
     * @param {int} group.GradeNum
     * @returns {string}
     */
    let openGroupContainer = (group) => {
        return `<div class="GroupContainerAttached" id="${group.GroupID}">
            <div class='groupInfo'><div class="groupInfoAlone">${group.GradeNum}.${group.GroupNum}</div></div>`;
    };

    let closeGroupContainer = () => {
        return `</div>`;
    };
    /**
     * @param {object} student
     * @param {int} student.ID
     * @param {string} student.LastName
     * @param {string} student.FirstName
     * @param {string} student.SecondName
     * @returns {*}
     */
    let constructUnbindStudent = (student) => {
        return `<div id="${student.ID}" class="Student StatusUnbind">
            <span class='Name'>${student.LastName} ${student.FirstName} ${student.SecondName}</span>
            <span id='0' class='From'></span>
            <button class='action Action_BindStudent'>Прикрепить студента</button>
        </div>`;
    };

    function SearchStudents() {
        let Name = $studentInput.val();
        let GradeID = +$gradeSelect.val();
        let GroupID = +$groupSelect.val();
        let params = {GradeID, GroupID, FacultyID, Name, DisciplineID};

        $.postJSON(g_URLdir + 'handler/map/SearchStudents', params, (searchResult) => {
            if (searchResult.length <= 0) {
                let msg = `Нет результатов... Возможно, студенты, соответствующие критериям поиска, уже прикреплены.`;
                $searchResult.html(`<p class="notification">${msg}</p>`);
                return;
            }

            let lastGroup = searchResult[0];
            let studentsHtml = openGroupContainer(lastGroup);

            searchResult.forEach((student) => {
                if (student.GroupID != lastGroup.GroupID) {
                    lastGroup = student;
                    studentsHtml += closeGroupContainer();
                    studentsHtml += openGroupContainer(lastGroup);
                }
                studentsHtml += constructUnbindStudent(student);
            });
            studentsHtml += closeGroupContainer();
            $searchResult.html(studentsHtml);
        }); //!post
    }

    var updateList = () => {
        SearchStudents();
        $(this).blur();
    };

    $('.searchBtn').click(updateList);

    $studentInput
        .keyup(updateList)
        .bind('paste', updateList);


    let groupSelect = getGroupLoader($groupSelect,
        () => ({'FacultyID': g_facultyID, 'GradeID': +$gradeSelect.val()})
    ).setOnChange(OptionLoader.stubFoo);

    $gradeSelect.change(() => groupSelect.reload());


    // ============================
    //      students lists
    // ============================

    // Показать/скрыть список студентов
    $studentsList.on('click', '.ActionShowHideGroupContainer', function(){
        let $groupContainer = $(this).next('.GroupContainer');
        if ($groupContainer.css('display') == 'none') {
            $(this).children('.Action').text('Скрыть список ▲');
            $groupContainer.show();
            $groupContainer.height($groupContainer.height() + 3);
        } else {
            $(this).children('.Action').text('Открыть список ▼');
            $groupContainer.hide('normal');
        }
    });

    $studentsList.on('click', 'div.hideListAction', function(){
        $(this).parent().hide('normal');
        $(this).parent().prev().children('.Action').text('Открыть список ▼');
    });


    // ============================
    //      students attachment
    // ============================

    let setBindStatus = ($this) => {
        $this
            .removeClass().addClass('action Action_UnbindStudent')
            .text('Отсоединить студента');
        $this.parent()
            .removeClass().addClass('Student StatusBind');
    };

    // Если студент из поиска
    let r = /([0-9]+).([0-9]+)/;
    let bindStudentFromSearch = (jThis) => {
        let $studentRecord = jThis.parent();
        let $groupContainer = $studentRecord.parent();
        let GroupID = +$groupContainer.attr('id');

        let $group = $attStudentList.children(`#${GroupID}`);
        if ($group.length <= 0) {
            r.lastIndex = 0;
            let [, GradeNum, GroupNum] = r.exec($groupContainer.children().html());
            $group = $(openGroupContainer({GroupID, GradeNum, GroupNum}));
            $attStudentList.append($group);
        }

        var len = $groupContainer.children().length;
        if (len <= 1)
            $groupContainer.remove();
        else
            $groupContainer.height($groupContainer.height() - 40);

        setBindStatus($studentRecord.children('button.action'));
        $group.height($group.height() + 40);
        $group.append($studentRecord);
    };



    let bindStudentHandler = (success) => {
        return function() { // need `this`!
            $(this).turnOff();
            let StudentID = +$(this).parent().attr('id');

            $.postJSON(`${URLdir}handler/map/BindStudent`, {StudentID, DisciplineID}, (data) => {
                if(data.success === true)
                    success($(this));
                else
                    Popup.error('Ошибка при добавлении студента');
                $(this).turnOn();
            });
        };
    };

    let unbindStudentHandler = function(success) {
        return function() { // need `this`!
            $(this).turnOff();
            var StudentID = $(this).parent().attr('id');

            $.postJSON(`${URLdir}handler/map/UnbindStudent`, {StudentID, DisciplineID}, (data) => {
                if (data.success === true)
                    success($(this));
                else
                    Popup.error('Ошибка при отсоединении студента');
                $(this).turnOn();
            });
        };
    };

    // Прикрепить студента
    $generalStudentsList.on('click', 'button.Action_BindStudent', bindStudentHandler(setBindStatus));

    // Прикрепить студента из поиска
    $studentsList.on('click', 'button.Action_BindStudent', bindStudentHandler(bindStudentFromSearch));

    // Отсоединить студента
    $generalStudentsList.on('click', 'button.Action_UnbindStudent', unbindStudentHandler(($this) => {
        $this
            .removeClass('Action_UnbindStudent')
            .addClass('Action_BindStudent')
            .text('Прикрепить студента');
        $this
            .parent()
            .removeClass('StatusBind')
            .addClass('StatusUnbind');
    }));

    $('.AttachedStudentsList').on('click', 'button.Action_UnbindStudent', unbindStudentHandler(($this) => {
        let $parent = $this.parent().parent();
        let len = $parent.children().length;

        $this.parent().remove();
        if (len <= 1)
            $parent.remove();
        else
            $parent.css('height', $parent.height() - 40);
    }));

});
