var $ = jQuery;

$(function () {
    var R = Rating();

    let optionsViewData = {
        'absence': {
            sendingOption: 'absence',
            messageOK: 'Неявка выставлена',
            messageFail: 'Установка неявки не удалась'
        },
        'absence_unset': {
            sendingOption: null,
            messageOK: 'Неявка отменена',
            messageFail: 'Отмена неявки не удалась'
        },
        'pass': {
            sendingOption: 'pass',
            messageOK: 'Автомат выставлен',
            messageFail: 'Установка автомата не удалась'
        },
        'pass_unset': {
            sendingOption: null,
            messageOK: 'Автомат отменен',
            messageFail: 'Отмена автомата не удалась'
        }
    };

    var StudentState = {
        successful: 0, // успевающий
        beforeExam: 1, // экзамен еще не состоялся
        notPassed: 2, // пытался сдать экзамен и не сдал
        notAdmitted: 3, // недопущенный задолженник
        admitted: 4, // допуск получен, экзамен еще не состоялся
        admittedAndPassed: 5, // задолженность закрыта полностью
        admittedAndNotPassed: 6 // допуск получен, экзамен не сдал
    };

    (function () {
        $('.studentsRate').children().children('tr')
            .each(function () {
                let id = $(this).attr('id');
                if (id && id.substr(0, 3) === 'row')
                    controlRowVisualization($(this));
            });
    })();

    R.adjuster.autoResize();
    R.filter.listen($('.groupSelector'));
    R.initGeneralCells($this => {
        controlRowVisualization($this.parent());
    });


    function deriveStudentState(semesterRate, extraRate, examRate, autopass, firstAttempt) {
        var admissionRate = semesterRate + extraRate;

        if (admissionRate >= 38) {
            if ((semesterRate >= 38) && firstAttempt) {
                if ((examRate >= 22) || autopass)
                    return StudentState.successful; // успевающий
                else if (examRate == -1)
                    return StudentState.beforeExam; // экзамен еще не состоялся
                else
                    return StudentState.notPassed; // пытался сдать экзамен и не сдал
            } else {
                if (examRate == -1)
                    return StudentState.admitted; // допуск получен, экзамен еще не состоялся
                else if (examRate >= 22)
                    return StudentState.admittedAndPassed; // задолженность закрыта
                else
                    return StudentState.admittedAndNotPassed; // допуск получен, экзамен не сдал
            }
        } else
            return StudentState.notAdmitted; // недопущенный задолженник
    }

    function setCellAccess(cell, access) {
        access ? $(cell).turnOn() : $(cell).turnOff();
    }

    function controlRowVisualization(jRow) {
        if (R.settings.Type !== 'exam')
            return;

        var jAutoPassCheckBox = jRow.children('.autoPass').children('.autoPassCheck');
        var jAbsenceCheckBoxes = jRow.children('.absenceCell').children('.absenceCheck');
        var jAttemptCells = jRow.children('.attemptCell').find(':text').parent();
        var jExtraInputs = jRow.children('.additionalCell').children('.extra');
        var semesterRate = parseInt(jRow.children('.semesterRateResultCell').text());
        var autopass = jAutoPassCheckBox[0].checked;
        var absence = jAbsenceCheckBoxes[0].checked;

        if (semesterRate < 60 || absence)
            jAutoPassCheckBox.turnOff();
        else
            jAutoPassCheckBox.turnOn();

        // суммарный добор
        var extraRate = jExtraInputs.aggregate((cur, elem) => +$(elem).val() + cur);
   
        // Определим экзамен
        var curExamCell = 0;
        var curExamRate = 0;
        var curExamCellIndex = 0;
        var index = 0;
        var dependentCells = [[0, 0], [0, 0], [0, 0]]; // размер зависит от jAttemptCells.length
        jAttemptCells.each(function () {
            var needToRemeber = (curExamCell == 0); // инициализация первым экзаменом

            let [col, row] = Base.parsePosition($(this));
            var neighborAbsenceCheck = $($(`#absence_${col}_${row}`).children()[0]);

            dependentCells[index][0] = $(this);
            dependentCells[index][1] = neighborAbsenceCheck.parent();

            var strVal = $(this).children()[0].value;
            var examRate = +strVal;
            if (strVal === '') {
                examRate = -1; // спец значение, означает 'не было оценки за экзамен'
                if (neighborAbsenceCheck.is(':checked')) {// неявка = 0 баллов
                    examRate = 0;
                    needToRemeber = true;
                }
            } else
                needToRemeber = true;

            if (needToRemeber) {
                curExamCell = $(this);
                curExamRate = examRate;
                curExamCellIndex = index;
            }

            index++;
        });

        var state = deriveStudentState(semesterRate, extraRate, curExamRate, autopass, curExamCellIndex == 0);

        // Выставляем доступность ячеек

        // Доступность добора
        var needAccess = !(state == StudentState.successful ||
        state == StudentState.beforeExam ||
        state == StudentState.notPassed);
        setCellAccess($(jExtraInputs[0]), needAccess);

        // Доступность экзаменов
        var examAccess = [false, false, false];
        if (state == StudentState.successful || state == StudentState.beforeExam)
            examAccess[0] = true; // только основная сдача
        else if (state == StudentState.admitted)
            examAccess[1] = true; // только пересдача 1
        else if (state == StudentState.admittedAndPassed)
            examAccess[curExamCellIndex] = true; // только текущая пересдача
        else if (state == StudentState.notPassed || state == StudentState.admittedAndNotPassed) {
            examAccess[curExamCellIndex] = true; // текущая сдача и следующий
            if (++curExamCellIndex < 3) // 3 == jAttemptCells.length
                examAccess[curExamCellIndex] = true;
        }

        for (index = 0; index < 3; ++index) { // 3 == jAttemptCells.length
            var examCell = dependentCells[index][0];
            var absenceCell = dependentCells[index][1];
            var absenceChecked = absenceCell.children()[0].checked;

            needAccess = examAccess[index] && !absenceChecked;
            setCellAccess(examCell.children()[0], needAccess);
            needAccess = examAccess[index]; //&& (val == 0);
            setCellAccess(absenceCell.children()[0], needAccess);
        }
    }


    // ================================================
    //          exam page specific
    // ================================================

    // Нажатие на чекбокс "Неявка"
    $('.absenceCheck').click(function () {
        // prepare
        var option = 'absence';
        var checked = $(this)[0].checked;
        if (!checked)
            option += '_unset';
        var thisTd = $(this).parent();
        R.cursor.parseLocation(thisTd);

        // call
        R.rateInfo.show(thisTd);
        var errCode = setExamPeriodOption($(this), option);
        R.rateInfo.hide(thisTd);

        // test
        if (errCode > 0)
            $(this).prop('checked', !checked); // invert check;
        controlRowVisualization(thisTd.parent());
    });

    $('.autoPassCheck').click(function () {
        // prepare
        var option = 'pass';
        var checked = $(this)[0].checked;
        if (!checked)
            option += '_unset';
        var thisTd = $(this).parent();
        R.cursor.parseLocation(thisTd);

        // call
        R.rateInfo.show(thisTd);
        var errCode = setExamPeriodOption($(this), option);
        R.rateInfo.hide(thisTd);

        // test
        if (errCode > 0)
            $(this).prop('checked', !checked); // invert check;
        controlRowVisualization(thisTd.parent());
    });

    function setExamPeriodOption($this, option) {
        $this.turnOff();

        $.postJSON(URLdir + 'handler/rating/SetExamPeriodOption',
            {
                studentID: R.cell.student,
                submoduleID: R.cell.submodule,
                option: '' + optionsViewData[option].sendingOption // cast null to "null"
            })
        .done(data => EventInspector.success(optionsViewData[option].messageOK))
        .fail(jqXHR => {
            $this.prop("checked", !$this.prop("checked"));

            let status = +jqXHR.status;
            let errorMsg =  (status == 400) ? optionsViewData[option].messageFail :
                            (status == 403) ? 'Сессия истекла' : '' + jqXHR.status; // null to 'null'
            Popup.error(errorMsg);

            if (status === 403)
                window.location.replace(URLdir);

        }).always(() => $this.turnOn());
    }

    // Скачать таблицу оценивания в формате excel // depricated
    $('.downloadExcel').click(function () {
        $.fileDownload(URLdir + 'handler/FileCreator/GenerateExcelRatingTable', {
            httpMethod: 'POST',
            data: {'disciplineID': R.settings.disciplineID},
            successCallback: function () {},
            failCallback: function () {}
        });
    });

    // Ведомость в формате excel
    $('body').on('click', '.downloadExcelStatement', function () {

        var groupID = parseID($(this));
        var $stageSelector = $('#stageSelector_' + groupID);

        $.fileDownload(URLdir + 'handler/FileCreator/GenerateFinalForm', {
            httpMethod: 'POST',
            data: {
                'disciplineID': R.settings.disciplineID,
                'groupID': groupID,
                'stage': parseInt($stageSelector.val())
            },
            successCallback: function () {},
            failCallback: function () {}
        });
    });
});
